/* $Id: td_local.h,v 12.5 1995/02/11 19:21:04 tom Exp $ */

/*
 * Include all features in TD_LIB, used for configuration.
 */
#define	ACC_PTYPES
#define	CHR_PTYPES
#define	CUR_PTYPES
#define	DIR_PTYPES
#define	GRP_PTYPES
#define	OPN_PTYPES
#define	PWD_PTYPES
#define	SIG_PTYPES
#define	STR_PTYPES
#define	TIM_PTYPES
#define	TRM_PTYPES
#define	WAI_PTYPES

#define TESTING_CONFIG_H

#include <ptypes.h>
#include <td_qsort.h>
#include <td_scomp.h>
#include <dyn_str.h>
#include <cmv_defs.h>
#include <rcsdefs.h>
#include <sccsdefs.h>
#include <td_sheet.h>
#include <td_lib.h>
#include <port2vms.h>
